use axum::Json;
use serde::Serialize;
use utoipa::ToSchema;

pub mod channel;
pub mod comment;
pub mod middleware;
pub mod video;

#[derive(Serialize, ToSchema)]
pub struct IndexResponse {
    app_name: String,
    version: String,
}

/// Get general information from the current Almond instance
#[utoipa::path(get, path = "/", tags = ["application"], responses((status = 200, body = IndexResponse)))]
pub async fn index() -> Json<IndexResponse> {
    let app_name = "Almond".into();
    let version = env!("CARGO_PKG_VERSION").into();

    Json(IndexResponse { app_name, version })
}
